@@ -24,9 +24,10 @@
 #define GOFFICE_GCHEMUTILS_PRIV_H
 
 #include "gchemutils.h"
+#include <gcu/structs.h>
 #include <gogcuapp.h>
 #include <gcu/document.h>
-#include <gcu/window.h>
+#include <gcugtk/window.h>
 
 struct _GOGChemUtilsComponent
 {
@@ -34,7 +35,8 @@ struct _GOGChemUtilsComponent
 
 	GOGcuApplication *application;
 	gcu::Document *document;
-	gcu::Window *window; // TODO use a gcu::Window.
+	gcugtk::Window *window; // TODO use a gcu::Window.
+	gcu::ContentType type;
 };
 
 
