@@ -51,37 +51,29 @@ void GOGcpApplication::ToggleMenu (G_GNUC_UNUSED const string& menuname, G_GNUC_
 {
 }
 
-gcu::Document *GOGcpApplication::ImportDocument (const string& mime_type, const char* data, int length)
+void GOGcpApplication::ImportDocument (GOGChemUtilsComponent *gogcu)
 {
 	gcp::Document *pDoc = NULL;
-	char *old_num_locale, *old_time_locale;
-	if (mime_type == "application/x-gchempaint") {
+	GOComponent *component = GO_COMPONENT (gogcu);
+	if (!strcmp (component->mime_type, "application/x-gchempaint")) {
 		xmlDocPtr xml;
-		if (!(xml = xmlParseMemory (data, length)) ||
+		if (!(xml = xmlParseMemory (component->data, component->length)) ||
 			xml->children == NULL ||
 			strcmp(reinterpret_cast <char const *> (xml->children->name), "chemistry")) {
 			xmlFreeDoc (xml);
-			return NULL;
+			return;
 		}
-		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
-		old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
-		setlocale (LC_TIME, "C");
 		pDoc = new gcp::Document (this, false);
 		pDoc->GetView ()->CreateNewWidget ();
 		bool result = pDoc->Load (xml->children);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
-		setlocale (LC_TIME, old_time_locale);
-		g_free (old_time_locale);
 		xmlFreeDoc (xml);
 		if (!result) {
 			delete pDoc;
-			return NULL;
+			return;
 		}
 	} else {
 	}
-	return (pDoc);
+	gogcu->document = pDoc;
 }
 
 GtkWindow * GOGcpApplication::EditDocument (GOGChemUtilsComponent *gogcu)
