@@ -1,11 +1,11 @@
-/* 
+/*
  * GChemPaint library
- * fontsel.c 
+ * fontsel.c
  *
- * Copyright (C) 2006-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -22,14 +22,7 @@
 
 #include "config.h"
 #include "fontsel.h"
-#include <gtk/gtkbin.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtk.h>
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 #include <cstdlib>
@@ -66,28 +59,6 @@ typedef struct {
 	void (*changed) (GcpFontSel *fs);
 } GcpFontSelClass;
 
-static void
-gcp_font_sel_size_request (GtkWidget      *widget,
-			GtkRequisition *requisition)
-{
-	GtkWidget *w = GTK_WIDGET (gtk_bin_get_child (GTK_BIN (widget)));
-	if (w)
-    	gtk_widget_size_request (w, requisition);
-	else {
-		requisition->width = 0;
-		requisition->height = 0;
-	}
-}
-
-static void
-gcp_font_sel_size_allocate (GtkWidget     *widget,
-			 GtkAllocation *allocation)
-{
-	GtkWidget *w = GTK_WIDGET (gtk_bin_get_child (GTK_BIN (widget)));
-	if (w)
-		gtk_widget_size_allocate (GTK_WIDGET (w), allocation);
-}
-
 enum {
 	FONT_SEL_PROP_0,
 	FONT_SEL_PROP_FAMILY,
@@ -223,7 +194,7 @@ static void gcp_font_sel_set_size_full (GcpFontSel *fs, bool update_list)
 		GtkTreeIter iter;
 		bool found = false;
 		g_signal_handler_block (fs->SizeSel, fs->SizeSignal);
-		
+
 		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (fs->SizeList), &iter);
 		for (unsigned i = 0; i < G_N_ELEMENTS (font_sizes) && !found; i++) {
 			if (font_sizes[i] * PANGO_SCALE == fs->Size) {
@@ -235,7 +206,7 @@ static void gcp_font_sel_set_size_full (GcpFontSel *fs, bool update_list)
 
 			gtk_tree_model_iter_next (GTK_TREE_MODEL (fs->SizeList), &iter);
 		}
-		
+
 		if (!found)
 			gtk_tree_selection_unselect_all (fs->SizeSel);
 		g_signal_handler_unblock (fs->SizeSel, fs->SizeSignal);
@@ -259,7 +230,7 @@ gcp_font_sel_set_property (GObject *obj, guint param_id,
 		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (fs->FamilyList), &iter);
 		do {
 			gtk_tree_model_get (GTK_TREE_MODEL (fs->FamilyList), &iter, 0, &buf, -1);
-			if (!strcmp (fs->FamilyName, buf)) {				
+			if (!strcmp (fs->FamilyName, buf)) {
 				GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (fs->FamilyList), &iter);
 				gtk_tree_view_set_cursor (fs->FamilyTree, path, NULL, FALSE);
 				gtk_tree_view_scroll_to_cell (fs->FamilyTree, path, NULL, FALSE, 0., 0.);
@@ -327,6 +298,41 @@ gcp_font_sel_set_property (GObject *obj, guint param_id,
 	select_best_font_face (fs);
 }
 
+static void gcp_font_sel_get_preferred_height (GtkWidget *w, gint *minimum_height, gint *natural_height)
+{
+	GtkWidget *child = gtk_bin_get_child (GTK_BIN (w));
+	gboolean visible = FALSE;
+	if (child)
+		g_object_get (G_OBJECT (child), "visible", &visible, NULL);
+	if (visible)
+		gtk_widget_get_preferred_height (child, minimum_height, natural_height);
+	else
+		*minimum_height = *natural_height = 0;
+}
+
+static void gcp_font_sel_get_preferred_width (GtkWidget *w, gint *minimum_width, gint *natural_width)
+{
+	GtkWidget *child = gtk_bin_get_child (GTK_BIN (w));
+	gboolean visible = FALSE;
+	if (child)
+		g_object_get (G_OBJECT (child), "visible", &visible, NULL);
+	if (visible)
+		gtk_widget_get_preferred_width (child, minimum_width, natural_width);
+	else
+		*minimum_width = *natural_width = 0;
+}
+
+static void gcp_font_sel_size_allocate (GtkWidget *w, GtkAllocation *alloc)
+{
+	GtkWidget *child = gtk_bin_get_child (GTK_BIN (w));
+	gboolean visible = FALSE;
+	if (child)
+		g_object_get (G_OBJECT (child), "visible", &visible, NULL);
+	if (visible)
+		gtk_widget_size_allocate (child, alloc);
+	(GTK_WIDGET_CLASS (font_sel_parent_class))->size_allocate (w, alloc);
+}
+
 static void
 gcp_font_sel_finalize (GObject *obj)
 {
@@ -339,15 +345,13 @@ static void
 gcp_font_sel_class_init (GcpFontSelClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass*) klass;
-	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+	GtkWidgetClass *widget_class = reinterpret_cast < GtkWidgetClass * > (klass);
 	font_sel_parent_class = (GObjectClass*) g_type_class_peek_parent (klass);
 
 	object_class->get_property = gcp_font_sel_get_property;
 	object_class->set_property = gcp_font_sel_set_property;
 	object_class->finalize = gcp_font_sel_finalize;
 
-	widget_class->size_request = gcp_font_sel_size_request;
-	widget_class->size_allocate = gcp_font_sel_size_allocate;
 	g_object_class_install_property (object_class, FONT_SEL_PROP_FAMILY,
 		g_param_spec_string ("family", _("Family"),
 			_("Font family"),
@@ -393,6 +397,9 @@ gcp_font_sel_class_init (GcpFontSelClass *klass)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+	widget_class->get_preferred_height = gcp_font_sel_get_preferred_height;
+	widget_class->get_preferred_width = gcp_font_sel_get_preferred_width;
+	widget_class->size_allocate = gcp_font_sel_size_allocate;
 }
 
 static void on_select_family (GtkTreeSelection *selection, GcpFontSel *fs)
@@ -503,18 +510,19 @@ gcp_font_sel_init (GcpFontSel *fs)
 {
 	int i, nb;
 	PangoFontFamily **families;
-	GtkWidget *sc, *w = gtk_table_new (3, 4, FALSE);
-	g_object_set (G_OBJECT (w), "border-width", 6, NULL);
+	GtkWidget *sc, *w = gtk_grid_new ();
+	g_object_set (G_OBJECT (w), "border-width", 6, "expand", true, NULL);
 	fs->Families = map<string, PangoFontFamily*>();
 	fs->Faces = map<string, PangoFontFace*>();
-	GtkTable *table = GTK_TABLE (w);
-	gtk_table_set_col_spacings (table, 12);
+	GtkGrid *grid = GTK_GRID (w);
+	if (gtk_check_version (3, 2, 0))
+		gtk_grid_set_row_spacing (grid, 6);
+	else
+		gtk_grid_set_column_spacing (grid, 6);
 	gtk_container_add (GTK_CONTAINER (fs), GTK_WIDGET (w));
 	w = gtk_label_new ("");
 	fs->Label = GTK_LABEL (w);
-	gtk_table_attach (table, w, 0, 3, 3, 4,
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 6);
+	gtk_grid_attach (grid, w, 0, 3, 3, 1);
 	// Initialize faces list
 	fs->FaceList = gtk_list_store_new (1, G_TYPE_STRING);
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (fs->FaceList), 0, GTK_SORT_ASCENDING);
@@ -523,9 +531,8 @@ gcp_font_sel_init (GcpFontSel *fs)
 	sc = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sc), GTK_WIDGET (fs->FacesTree));
-	gtk_table_attach (table, sc, 1, 2, 1, 3,
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	g_object_set (sc, "expand", true, NULL);
+	gtk_grid_attach (grid, sc, 1, 1, 1, 2);
 	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (fs->FacesTree, column);
@@ -553,17 +560,14 @@ gcp_font_sel_init (GcpFontSel *fs)
 	sc = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sc), w);
-	gtk_table_attach (table, sc, 2, 3, 2, 3,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_grid_attach (grid, sc, 2, 2, 1, 1);
 	w = gtk_entry_new ();
+	gtk_entry_set_width_chars (GTK_ENTRY (w), 4);
 	fs->m_SizeEntry = GTK_ENTRY (w);
 	g_signal_connect (w, "activate", G_CALLBACK (on_size_activate), fs);
 	g_signal_connect_after (w, "focus_out_event", G_CALLBACK (on_size_focus_out), fs);
 	gcp_font_sel_set_size_full (fs, true);
-	gtk_table_attach (table, w, 2, 3, 1, 2,
-			(GtkAttachOptions) (0),
-			(GtkAttachOptions) 0, 0, 0);
+	gtk_grid_attach (grid, w, 2, 1, 1, 1);
 	PangoContext *pc = gtk_widget_get_pango_context (w);
 	PangoLayout *pl = pango_layout_new (pc);
 	pango_layout_set_text (pl, "0000000", -1);
@@ -608,11 +612,10 @@ gcp_font_sel_init (GcpFontSel *fs)
 	gtk_tree_selection_set_mode (fs->FamilySel, GTK_SELECTION_BROWSE);
 	fs->FamilySignal = g_signal_connect (G_OBJECT (fs->FamilySel), "changed", G_CALLBACK (on_select_family), fs);
 	sc = gtk_scrolled_window_new (NULL, NULL);
+	g_object_set (sc, "expand", true, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sc), GTK_WIDGET (fs->FamilyTree));
-	gtk_table_attach (table, sc, 0, 1, 1, 3,
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_grid_attach (grid, sc, 0, 1, 1, 2);
 	g_free (families);
 	fs->AllowSlanted = true;
 }
