@@ -1,13 +1,13 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * fragment.cc 
+ * fragment.cc
  *
  * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -356,7 +356,7 @@ bool Fragment::OnChanged (bool save)
 	pView->Update (this);
 	m_bLoading = false;
 	m_Valid = Invalid;
-	Window* pWin = pDoc->GetWindow ();
+	Window *pWin = static_cast < Window * > (pDoc->GetWindow ());
 	if (m_Atom->GetZ () || ((m_buf.length () == 0) && (m_Atom->GetBondsNumber () == 0))) {
 		if (!pDoc->GetReadOnly ()) {
 			pWin->ActivateActionWidget ("/MainMenu/FileMenu/Save", true);
@@ -522,7 +522,7 @@ void Fragment::UpdateItem ()
 void Fragment::SetSelected (int state)
 {
 	GOColor color, othercolor = 0;
-	switch (state) {	
+	switch (state) {
 	case SelStateUnselected:
 		color = GO_COLOR_BLACK;
 		break;
@@ -599,7 +599,7 @@ bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsi
 	for (j = tags->begin (); j != jend; j++) {
 		if (end <= (*j)->GetStartIndex () || start >= (*j)->GetEndIndex ())
 			continue;
-		if (start < (*j)->GetStartIndex ()) 
+		if (start < (*j)->GetStartIndex ())
 			xmlNodeAddContentLen (node, reinterpret_cast <xmlChar const *> (m_buf.c_str () + start), (*j)->GetStartIndex () - start);
 		gccv::Tag tag = (*j)->GetTag ();
 		if (tag == gccv::Position) {
@@ -865,7 +865,7 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 				}
 				if ((*i)->GetTag () == StoichiometryTag) {
 					tag =*i;
-					Stoich = true; 
+					Stoich = true;
 					break;
 				}
 			}}
@@ -1177,7 +1177,7 @@ Object* Fragment::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 		m_x += m_lbearing / pTheme->GetZoomFactor ();
 		m_Atom->SetCoords(m_x, m_y);
 	}
-	
+
 	return m_Atom;
 }
 
@@ -1338,10 +1338,10 @@ bool Fragment::Validate ()
 		&& m_Atom->GetBondsNumber () == 0)
 		return true;
 	if (m_Atom->GetZ() == 0 || (dynamic_cast <FragmentResidue*> (m_Atom) && !((FragmentResidue*) m_Atom)->GetResidue ())) {
-		Document *pDoc = dynamic_cast<Document*> (GetDocument ());
+		Document *pDoc = dynamic_cast <Document*> (GetDocument ());
 		m_TextItem->SetSelectionBounds (m_BeginAtom, (m_EndAtom == m_BeginAtom)? m_EndAtom + 1: m_EndAtom);
 		GtkWidget* w = gtk_message_dialog_new (
-										GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
+										GTK_WINDOW (static_cast < Window *> (pDoc->GetWindow ())->GetWindow ()),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
 										GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 										_("Invalid symbol."));
@@ -1370,7 +1370,7 @@ bool Fragment::Validate ()
 			Document *pDoc = dynamic_cast<Document*> (GetDocument ());
 			m_TextItem->SetSelectionBounds ((*it)->GetStartIndex (), (*it)->GetEndIndex ());
 			GtkWidget* w = gtk_message_dialog_new (
-											GTK_WINDOW (pDoc->GetWindow ()->GetWindow ()),
+											GTK_WINDOW (static_cast <Window * > (pDoc->GetWindow ())->GetWindow ()),
 											GTK_DIALOG_DESTROY_WITH_PARENT,
 											GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 											_("Invalid charge."));
@@ -1394,7 +1394,7 @@ void Fragment::Transform2D (Matrix2D& m, double x, double y)
 	m_y += y;
 	m_Atom->SetCoords (m_x, m_y);
 }
-	
+
 double Fragment::GetYAlign ()
 {
 	return m_y;
