@@ -28,6 +28,7 @@
 #include <goffice/goffice.h>
 #include <glib/gi18n-lib.h>
 #include <list>
+#include <set>
 #include <string>
 #include <vector>
 #include <cstring>
@@ -41,10 +42,11 @@ struct _GcrGrid
 {
 	GtkLayout base;
 	unsigned cols, rows, allocated_rows;
-	int col, row;
+	int col, row, last_row;
 	int first_visible;
 	unsigned nb_visible;
-	int header_width, row_height, width, *col_widths, line_offset, scroll_width, *min_widths, cols_min_width;
+	int header_width, row_height, width, actual_width, *col_widths,
+		line_offset, scroll_width, *min_widths, cols_min_width;
 	int cursor_index, sel_start;
 	GtkAdjustment *vadj;
 	GtkWidget *scroll;
@@ -56,6 +58,10 @@ struct _GcrGrid
 	unsigned long cursor_signal;
 	std::string *orig_string;
 	int can_edit;
+	bool allow_multiple;
+	bool dragging;
+	bool selection_locked;
+	std::set < int > *selected_rows;
 };
 
 static GtkWidgetClass *parent_class;
@@ -64,6 +70,7 @@ static GdkPixbuf *checked = NULL, *unchecked = NULL;
 enum {
 	VALUE_CHANGED,
 	ROW_SELECTED,
+	ROW_DELETED,
 	LAST_SIGNAL
 };
 
@@ -220,7 +227,9 @@ static gboolean gcr_grid_draw (GtkWidget *w, cairo_t* cr)
 		cairo_rectangle (cr, 0, y, grid->header_width + 1, grid->row_height + 1);
 		cairo_fill (cr);
 		cairo_restore (cr);
-		gtk_style_context_set_state (ctxt, (row == grid->row)? GTK_STATE_FLAG_ACTIVE: GTK_STATE_FLAG_NORMAL);
+		gtk_style_context_set_state (ctxt, (row == grid->row ||
+		                                    grid->selected_rows->find (row) != grid->selected_rows->end ())?
+		                             		GTK_STATE_FLAG_ACTIVE: GTK_STATE_FLAG_NORMAL);
 		gtk_render_background (ctxt, cr, 0, y, grid->header_width + 1, grid->row_height + 1);
 		gtk_render_frame (ctxt, cr, 0, y, grid->header_width + 1, grid->row_height + 1);
 		char *buf = g_strdup_printf("%d", ++row);
@@ -236,12 +245,19 @@ static gboolean gcr_grid_draw (GtkWidget *w, cairo_t* cr)
 	cairo_rectangle (cr, grid->header_width, y, alloc.width - grid->header_width, alloc.height - y);
 	cairo_clip (cr);
 	// show a rectangle around current selection
-	if (grid->row >= 0 && grid->col >= 0) {
+	if (grid->row >= 0) {
 		pos = grid->header_width;
-		for (i = 0; static_cast < int > (i) < grid->col; i++)
-			pos += grid->col_widths[i];
 		cairo_save (cr);
-		cairo_rectangle (cr, pos + .5, y + (grid->row - grid->first_visible) * grid->row_height + .5, grid->col_widths[grid->col], grid->row_height);
+		if (grid->col >= 0) {
+			for (i = 0; static_cast < int > (i) < grid->col; i++)
+				pos += grid->col_widths[i];
+			cairo_rectangle (cr, pos + .5,
+			                 y + (grid->row - grid->first_visible) * grid->row_height + .5,
+			                 grid->col_widths[grid->col], grid->row_height);
+		} else
+				cairo_rectangle (cr, pos + .5,
+				                 y + (grid->row - grid->first_visible) * grid->row_height + .5,
+				                 grid->actual_width, grid->row_height);
 		cairo_set_line_width (cr, 3.);
 		cairo_stroke_preserve (cr);
 		cairo_restore (cr);
@@ -356,6 +372,7 @@ static void gcr_grid_finalize (GObject *obj)
 	delete [] grid->editable;
 	for (unsigned i = 0 ; i < grid->rows; i++)
 		delete [] grid->row_data[i];
+	delete grid->selected_rows;
 	reinterpret_cast < GObjectClass * > (parent_class)->finalize (obj);
 }
 
@@ -375,7 +392,8 @@ static void gcr_grid_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 	} else
 		gtk_adjustment_set_page_size (grid->vadj, 1.);
 	// adjust column widths
-	double ratio = static_cast < double > (alloc->width - grid->header_width - grid->scroll_width) / grid->cols_min_width;
+	grid->actual_width = alloc->width - grid->header_width - grid->scroll_width;
+	double ratio = static_cast < double > (grid->actual_width) / grid->cols_min_width;
 	if (ratio < 0.)
 		ratio = 1.;
 	double last_pos = 0., pos = 0.;
@@ -403,9 +421,12 @@ static gboolean gcr_grid_button_press_event (GtkWidget *widget, GdkEventButton *
 	if (event->button != 1)
 		return false;	// FIXME: at least middle buttons should be accepted to paste data
 	GcrGrid *grid = GCR_GRID (widget);
+	if ((event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) == 0)
+		grid->selected_rows->clear ();
 	int value_changed = -1;
 	int x = grid->first_visible + event->y / grid->row_height - 1, i, new_row, new_col;
 	new_row = (x < 0 || x >= static_cast < int > (grid->rows))? -1: x;
+	grid->dragging = true;
 	if (new_row < 0)
 		new_col = -1;
 	else {
@@ -423,12 +444,35 @@ static gboolean gcr_grid_button_press_event (GtkWidget *widget, GdkEventButton *
 	if (grid->col != new_col || grid->row != new_row) {
 		if (grid->row >= 0 && !gcr_grid_validate_change (grid))
 			return true;
-		if (!grid->editable[new_col])
+		if (grid->allow_multiple && event->state & GDK_SHIFT_MASK) {
 			new_col = -1;
-		if (new_row != grid->row)
-			g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, new_row);
-		grid->col = new_col;
-		grid->row = new_row;
+			if (new_row != grid->last_row) {
+				int incr = (grid->last_row > grid->row)? -1: 1;
+				for (x = grid->last_row; x != grid->row; x += incr)
+					grid->selected_rows->erase (x);
+				incr = (new_row > grid->row)? -1: 1;
+				for (x = new_row; x != grid->row; x += incr)
+					grid->selected_rows->insert (x);
+				grid->last_row = new_row;
+			}
+		} else if (grid->allow_multiple && event->state & GDK_CONTROL_MASK) {
+			new_col = -1;
+			if (grid->selected_rows->find (new_row) != grid->selected_rows->end ()) {
+				grid->selected_rows->erase (new_row);
+				new_row = grid->row;
+			} else
+				grid->selected_rows->insert (grid->row);
+				if (new_row != grid->row)
+					g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, new_row);
+				grid->row = grid->last_row = new_row;
+		} else {
+			if (!grid->editable[new_col])
+				new_col = -1;
+			if (new_row != grid->row)
+				g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, new_row);
+			grid->col = new_col;
+			grid->row = grid->last_row = new_row;
+		}
 	}
 	// evaluate the cursor position if any
 	if (grid->col >= 0) {
@@ -445,7 +489,10 @@ static gboolean gcr_grid_button_press_event (GtkWidget *widget, GdkEventButton *
 				xpos = event->x - startx;
 				int index, trailing;
 				pango_layout_xy_to_index (l, xpos * PANGO_SCALE, 0, &index, &trailing);
-				grid->sel_start = grid->cursor_index = index + trailing;
+				index += trailing;
+				grid->cursor_index = index;
+				if ((event->state & GDK_SHIFT_MASK) == 0)
+					grid->sel_start = index;
 			} else if (event->type == GDK_2BUTTON_PRESS) {
 				grid->sel_start = 0;
 				grid->cursor_index = grid->row_data[grid->row][grid->col].length ();
@@ -467,7 +514,6 @@ static gboolean gcr_grid_button_press_event (GtkWidget *widget, GdkEventButton *
 			break;
 		}
 		*grid->orig_string = grid->row_data[grid->row][grid->col];
-		gtk_widget_grab_focus (widget);
 	} else
 		grid->cursor_index = -1;
 	if (grid->cursor_index >= 0 && grid->cursor_signal == 0) {
@@ -479,10 +525,78 @@ static gboolean gcr_grid_button_press_event (GtkWidget *widget, GdkEventButton *
 	}
 	if (value_changed >= 0) // a boolean changed
 		g_signal_emit (grid, gcr_grid_signals[VALUE_CHANGED], 0, new_row, new_col);
+	gtk_widget_grab_focus (widget);
 	gtk_widget_queue_draw (widget);
 	return true;
 }
 
+static gboolean gcr_grid_motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
+{
+	GcrGrid *grid = GCR_GRID (widget);
+	if (!grid->dragging)
+		return true;
+	// evaluate the current row
+	int x = grid->first_visible + event->y / grid->row_height - 1, new_row, incr;
+	new_row = (x < 0 || x >= static_cast < int > (grid->rows))? -1: x;
+	// update selection
+	if (new_row != grid->last_row && grid->allow_multiple) {
+		incr = (grid->last_row > grid->row)? -1: 1;
+		for (x = grid->last_row; x != grid->row; x += incr)
+			grid->selected_rows->erase (x);
+		incr = (new_row > grid->row)? -1: 1;
+		for (x = new_row; x != grid->row; x += incr)
+			grid->selected_rows->insert (x);
+		grid->last_row = new_row;
+		grid->col = -1;
+	} else if (grid->col >= 0) {
+		int new_col = -1;
+		x = grid->header_width;
+		if (event->x >= x)
+			for (incr = 0; incr < static_cast < int > (grid->cols); incr++) {
+				x += grid->col_widths[incr];
+				if (event->x < x) {
+					new_col = incr;
+					break;
+				}
+			}
+		if (new_col == grid->col) {
+			// evaluate the cursor position if any
+			if (new_col >= 0) {
+				switch (grid->types[grid->col]) {
+				case G_TYPE_INT:
+				case G_TYPE_UINT:
+				case G_TYPE_DOUBLE: {
+					x -=  grid->col_widths[grid->col];
+					PangoLayout *l = gtk_widget_create_pango_layout (widget, grid->row_data[grid->row][grid->col].c_str());
+					int xpos, startx;
+					pango_layout_get_pixel_size (l, &xpos, NULL);
+					startx = x + (grid->col_widths[grid->col] - xpos) / 2;
+					xpos = event->x - startx;
+					int index, trailing;
+					pango_layout_xy_to_index (l, xpos * PANGO_SCALE, 0, &index, &trailing);
+					grid->cursor_index = index + trailing;
+					break;
+				}
+				default:	// nothing to do
+					break;
+				}
+				*grid->orig_string = grid->row_data[grid->row][grid->col];
+			}
+		} else
+			grid->col = -1;
+	}
+	gtk_widget_grab_focus (widget);
+	gtk_widget_queue_draw (widget);
+	return true;
+}
+
+static gboolean gcr_grid_button_release_event (GtkWidget *widget, G_GNUC_UNUSED GdkEventButton *event)
+{
+	GcrGrid *grid = GCR_GRID (widget);
+	grid->dragging = false;
+	return true;
+}
+
 static gboolean gcr_grid_scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
 	return gtk_widget_event (GCR_GRID (widget)->scroll, reinterpret_cast < GdkEvent * > (event));
@@ -493,7 +607,7 @@ static gboolean gcr_grid_key_press_event (GtkWidget *widget, GdkEventKey *event)
 	GcrGrid *grid = GCR_GRID (widget);
 	if (grid->row < 0)
 		return false;
-	int new_row = grid->row, new_col = grid->col, new_index = grid->cursor_index;
+	int new_row = grid->last_row, new_col = grid->col, new_index = grid->cursor_index;
 	char new_char = 0;
 	switch (event->keyval) {
 	case GDK_KEY_Tab:
@@ -537,16 +651,22 @@ static gboolean gcr_grid_key_press_event (GtkWidget *widget, GdkEventKey *event)
 			if ((event->state & GDK_SHIFT_MASK) == 0)
 				grid->sel_start = new_index;
 		} else {
-			do {
-				if (new_col > 0)
-					new_col--;
-				else if (new_row > 0) {
-					new_row--;
-					new_col = grid->cols - 1;
-				} else
-					return true;
-			} while (!grid->editable[new_col]);
-			new_index = grid->row_data[new_row][new_col].length ();
+			if (event->state & GDK_SHIFT_MASK) {
+				if (new_col < static_cast < int > (grid->cols) - 1)
+					new_col = -1; // select the whole line
+				grid->sel_start = new_index = -1;
+			} else {
+				do {
+					if (new_col > 0)
+						new_col--;
+					else if (new_row > 0) {
+						new_row--;
+						new_col = grid->cols - 1;
+					} else
+						return true;
+				} while (!grid->editable[new_col]);
+				new_index = grid->row_data[new_row][new_col].length ();
+			}
 		}
 		break;
 	case GDK_KEY_Right:
@@ -556,38 +676,91 @@ static gboolean gcr_grid_key_press_event (GtkWidget *widget, GdkEventKey *event)
 			if ((event->state & GDK_SHIFT_MASK) == 0)
 				grid->sel_start = new_index;
 		} else {
-			do {
+			if (event->state & GDK_SHIFT_MASK) {
 				if (new_col < static_cast < int > (grid->cols) - 1)
-					new_col++;
-				else if (grid->row < static_cast < int > (grid->rows) - 1) {
-					new_row++;
-					new_col = 0;
-				} else
-					return true;
-			} while (!grid->editable[new_col]);
-			new_index = 0;
+					new_col = -1; // select the whole line
+				grid->sel_start = new_index = -1;
+			} else {
+				do {
+					if (new_col < static_cast < int > (grid->cols) - 1)
+						new_col++;
+					else if (grid->row < static_cast < int > (grid->rows) - 1) {
+						new_row++;
+						new_col = 0;
+					} else
+						return true;
+				} while (!grid->editable[new_col]);
+				grid->selected_rows->clear ();
+				grid->sel_start = new_index = 0;
+			}
 		}
 		break;
 	case GDK_KEY_Up:
 	case GDK_KEY_KP_Up:
-		if (grid->row > 0)
+		if (event->state & GDK_SHIFT_MASK) {
+			new_col = -1; // select the whole line
+			grid->sel_start = new_index = -1;
+			if (new_row > 0) {
+				int row = new_row - 1;
+				if (row < grid->row)
+					gcr_grid_add_row_to_selection (grid, row);
+				else
+					gcr_grid_unselect_row (grid, grid->last_row);
+				grid->last_row = new_row = row;
+			}
+		} else if (grid->row > 0)
 			new_row--;
 		break;
 	case GDK_KEY_Down:
 	case GDK_KEY_KP_Down:
-		if (grid->row < static_cast < int > (grid->rows) - 1)
+		if (event->state & GDK_SHIFT_MASK) {
+			new_col = -1; // select the whole line
+			grid->sel_start = new_index = -1;
+			if (new_row < static_cast < int > (grid->rows) - 1) {
+				int row = new_row + 1;
+				if (row > grid->row)
+					gcr_grid_add_row_to_selection (grid, row);
+				else
+					gcr_grid_unselect_row (grid, grid->last_row);
+				grid->last_row = new_row = row;
+			}
+		} else if (grid->row < static_cast < int > (grid->rows) - 1)
 			new_row++;
 		break;
 	case GDK_KEY_Page_Up:
 	case GDK_KEY_KP_Page_Up:
-		if (grid->row >= static_cast < int > (grid->nb_visible))
+		if (event->state & GDK_SHIFT_MASK) {
+			new_col = -1; // select the whole line
+			grid->sel_start = new_index = -1;
+			int i, row = (grid->row >= static_cast < int > (grid->nb_visible))?
+							new_row - grid->nb_visible: 0;
+			for (i = row; i < new_row; i++) {
+				if (i < grid->row)
+					gcr_grid_add_row_to_selection (grid, i);
+				else
+					gcr_grid_unselect_row (grid, i);
+			}
+			grid->last_row = new_row = row;
+		} else if (grid->row >= static_cast < int > (grid->nb_visible))
 			new_row -= grid->nb_visible;
 		else
 			new_row = 0;
 		break;
 	case GDK_KEY_Page_Down:
 	case GDK_KEY_KP_Page_Down:
-		if (grid->rows > grid->nb_visible && grid->row < static_cast < int > (grid->rows - grid->nb_visible) - 1)
+		if (event->state & GDK_SHIFT_MASK) {
+			new_col = -1; // select the whole line
+			grid->sel_start = new_index = -1;
+			int i, row = (grid->rows > grid->nb_visible && grid->row < static_cast < int > (grid->rows - grid->nb_visible) - 1)?
+							new_row + grid->nb_visible: grid->rows - 1;
+			for (i = row; i > new_row; i--) {
+				if (i > grid->row)
+					gcr_grid_add_row_to_selection (grid, i);
+				else
+					gcr_grid_unselect_row (grid, i);
+			}
+			grid->last_row = new_row = row;
+		} if (grid->rows > grid->nb_visible && grid->row < static_cast < int > (grid->rows - grid->nb_visible) - 1)
 			new_row += grid->nb_visible;
 		else
 			new_row = grid->rows - 1;
@@ -805,28 +978,31 @@ static gboolean gcr_grid_key_press_event (GtkWidget *widget, GdkEventKey *event)
 		new_index++;
 		grid->sel_start = new_index;
 	}
-	if (new_row != grid->row || new_col != grid->col) {
-		if (grid->row >= 0 && !gcr_grid_validate_change (grid))
+	if (new_row != grid->last_row || new_col != grid->col) {
+		if (grid->row >= 0 && grid->col >= 0 && !gcr_grid_validate_change (grid))
 			return true;
 		if (new_row != grid->row)
 			g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, new_row);
-		if (grid->editable[new_col]) {
+		if (new_col >= 0 && grid->editable[new_col]) {
 			grid->row = new_row;
 			grid->col = new_col;
 			*grid->orig_string = grid->row_data[new_row][new_col];
 			int l = grid->orig_string->length ();
 			if (new_index > l)
 				new_index = grid->sel_start = l;
-		} else {
+		} else if (new_col >= 0) {
 			grid->row = grid->col = new_index = -1;
 			g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, -1);
-		}
+		} else
+			grid->col = new_col;
+		if ((event->state & GDK_SHIFT_MASK) == 0)
+			grid->last_row = grid->row;
 	}
 	// ensure that the selection is visible
-	if (new_row < grid->first_visible)
-		grid->first_visible = new_row;
-	else if (new_row >= grid->first_visible + static_cast < int > (grid->nb_visible))
-		grid->first_visible = new_row + 1 - grid->nb_visible;
+	if (grid->last_row < grid->first_visible)
+		grid->first_visible = grid->last_row;
+	else if (grid->last_row >= grid->first_visible + static_cast < int > (grid->nb_visible))
+		grid->first_visible = grid->last_row + 1 - grid->nb_visible;
 	grid->cursor_index = new_index;
 	gtk_widget_queue_draw (widget);
 	return true;
@@ -838,6 +1014,8 @@ static void gcr_grid_class_init (GtkWidgetClass *klass)
 	klass->draw = gcr_grid_draw;
 	klass->size_allocate = gcr_grid_size_allocate;
 	klass->button_press_event = gcr_grid_button_press_event;
+	klass->motion_notify_event = gcr_grid_motion_notify_event;
+	klass->button_release_event = gcr_grid_button_release_event;
 	klass->scroll_event = gcr_grid_scroll_event;
 	klass->key_press_event = gcr_grid_key_press_event;
 	klass->focus_out_event = gcr_grid_focus_out_event;
@@ -863,6 +1041,15 @@ static void gcr_grid_class_init (GtkWidgetClass *klass)
 				                                G_TYPE_NONE, 1,
 				                                G_TYPE_INT
 				                                );
+	gcr_grid_signals[ROW_DELETED] = g_signal_new ("row-deleted",
+	                                            G_TYPE_FROM_CLASS(klass),
+				                                G_SIGNAL_RUN_LAST,
+				                                G_STRUCT_OFFSET(GcrGridClass, row_deleted_event),
+				                                NULL, NULL,
+				                                g_cclosure_marshal_VOID__INT,
+				                                G_TYPE_NONE, 1,
+				                                G_TYPE_INT
+				                                );
 }
 
 static void
@@ -927,6 +1114,7 @@ GtkWidget *gcr_grid_new (G_GNUC_UNUSED char const *col_title, GType col_type, ..
 	grid->titles = new std::string[grid->cols];
 	grid->types = new GType[grid->cols];
 	grid->editable = new bool[grid->cols];
+	grid->selected_rows = new std::set < int > ();
 	unsigned i;
 	std::list <char const *>::iterator title = titles.begin ();
 	std::list <GType>::iterator type = types.begin ();
@@ -1136,16 +1324,52 @@ void gcr_grid_delete_row (GcrGrid *grid, unsigned row)
 {
 	g_return_if_fail (GCR_IS_GRID (grid) && grid->rows > row);
 	delete [] grid->row_data[row];
-	for (row++ ; row < grid->rows; row++)
-		grid->row_data[row - 1] = grid->row_data[row];
+	g_signal_emit (grid, gcr_grid_signals[ROW_DELETED], 0, row);
+	for (unsigned n = row + 1 ; n < grid->rows; n++)
+		grid->row_data[n - 1] = grid->row_data[n];
 	grid->rows--;
+	std::set < int > decreased;
+	std::set < int >::iterator i, end = grid->selected_rows->end ();
+	for (i = grid->selected_rows->begin (); i != end; i++)
+		if ((*i) > row)
+			decreased.insert (*i);
+	grid->selected_rows->erase (row);
+	end = decreased.end ();
+	for (i = decreased.begin (); i != end; i++)
+		grid->selected_rows->erase (*i);
+	for (i = decreased.begin (); i != end; i++)
+		grid->selected_rows->insert ((*i) - 1);
 	if (grid->row == static_cast < int > (grid->rows)) {
 		grid->row = -1;
 		g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, -1);
 	}
+	if (!grid->selection_locked)
+		grid->selected_rows->clear ();
 	gtk_widget_queue_draw (GTK_WIDGET (grid));
 }
 
+void gcr_grid_delete_selected_rows (GcrGrid *grid)
+{
+	g_return_if_fail (GCR_IS_GRID (grid));
+	if (grid->row == -1)
+		return;
+	// save row and set it to -1 to not fire ROW_SELECTED when not needed
+	int row = grid->row;
+	grid->row = -1;
+	grid->selection_locked = true;
+	gcr_grid_delete_row (grid, row);
+	while (!grid->selected_rows->empty ())
+		gcr_grid_delete_row (grid, *grid->selected_rows->begin ());
+	grid->selected_rows->clear ();
+	if (row >= static_cast < int > (grid->rows))
+		g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, -1);
+	else
+		grid->row = row;
+	g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, -1);
+	gtk_widget_queue_draw (GTK_WIDGET (grid));
+	grid->selection_locked = false;
+}
+
 void gcr_grid_delete_all (GcrGrid *grid)
 {
 	g_return_if_fail (GCR_IS_GRID (grid));
@@ -1183,3 +1407,50 @@ void gcr_grid_customize_column (GcrGrid *grid, unsigned column, unsigned chars,
 		gtk_widget_queue_resize (GTK_WIDGET (grid));
 	}
 }
+
+void gcr_grid_set_allow_multiple_selection (GcrGrid *grid, bool allow)
+{
+	grid->allow_multiple = allow;
+}
+
+void gcr_grid_for_each_selected (GcrGrid *grid, GridCb cb, void *user_data)
+{
+	g_return_if_fail (GCR_IS_GRID (grid));
+	if (grid->row < 0)
+		return;
+	cb (grid->row, user_data);
+	std::set < int >::iterator i, end = grid->selected_rows->end ();
+	for (i = grid->selected_rows->begin (); i != end; i++)
+		cb (*i, user_data);
+}
+
+void gcr_grid_select_all (GcrGrid *grid)
+{
+	g_return_if_fail (GCR_IS_GRID (grid)  && grid->allow_multiple);
+	if (grid->rows == 0)
+		return;
+	if (grid->row < 0) {
+		grid->row = 0;
+		g_signal_emit (grid, gcr_grid_signals[ROW_SELECTED], 0, 0);
+	} else if (grid->col > 0 && !gcr_grid_validate_change (grid))
+		return;
+	for (unsigned i = 0; i < grid->rows; i++)
+		if (i != static_cast < unsigned > (grid->row))
+			grid->selected_rows->insert (i);
+	gtk_widget_queue_draw (GTK_WIDGET (grid));
+}
+
+void gcr_grid_add_row_to_selection (GcrGrid *grid, unsigned row)
+{
+	if (grid->row < 0)
+		grid->row = row;
+	else if (row != static_cast < unsigned > (grid->row))
+		grid->selected_rows->insert (row);
+	gtk_widget_queue_draw (GTK_WIDGET (grid));
+}
+
+void gcr_grid_unselect_row (GcrGrid *grid, unsigned row)
+{
+	grid->selected_rows->erase (row);
+	gtk_widget_queue_draw (GTK_WIDGET (grid));
+}
