@@ -117,7 +117,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 	Application *app = doc->GetApplication ();
 	GsfInputTextline *input = reinterpret_cast <GsfInputTextline *> (gsf_input_textline_new (in));
 	char *buf;
-	bool in_string = false, in_loop = false, waiting_value = false;
+	bool in_string = false, in_loop = false, waiting_value = false, empty = true;
 	string key, value;
 	int size;
 	char endstr = 0;
@@ -137,7 +137,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 		size = strlen (buf);
 		// check for new data bloc
 		if (!strncmp (cur, "data_", 5)) {
-			if (!doc->GetEmpty ())
+			if (!empty)
 				// FIXME: implement multiple data blocs reading
 				goto read_exit;
 		}
@@ -278,6 +278,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 										} else {
 											LabeledAtoms[val] = atom;
 											doc->AddChild (atom);
+											empty = false;
 										}
 										if (AtomTypes.empty ()) {
 											int i = 0;
